==> sphere_connector-0.4.30.tar.gz: sphere_connector-0.4.30/sphere_connector/redcap/redcap_client.py <==
"""
The MIT License (MIT)
Copyright (c) 2014-2015 University of Bristol

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
OR OTHER DEALINGS IN THE SOFTWARE.
"""
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
from sqlalchemy import Column, Integer, String, Text
# from wtforms_alchemy import ModelForm
# from sshtunnel import SSHTunnelForwarder
import logging
from collections import defaultdict
from datetime import timedelta
import pytz
from dateutil.parser import parse

from ..utils import log_event
from ..config import ExperimentConfig

Base = declarative_base()


class RedCapData(Base):
    __tablename__ = 'redcap_data'
    project_id = Column(Integer, nullable=False, default=0, primary_key=True)
    event_id = Column(Integer, primary_key=True)
    record = Column(String(100), primary_key=True)
    field_name = Column(String(100), primary_key=True)
    value = Column(Text)

    def __repr__(self):
        return "<RedCapData(project_id='%s', event_id='%s', record='%s', field_name='%s', value='%s'>" % \
               (self.project_id, self.event_id, self.record, self.field_name, self.value)


# class RedCapDataForm(ModelForm):
#     class Meta:
#         model = RedCapData


class RedCapClient(object):
    def __init__(self, redcap_config):
        logging.debug(redcap_config)

        # if redcap_config.ssh:
        #     self.tunnel = SSHTunnelForwarder(
        #         (redcap_config.ssh['hostname'], redcap_config.ssh['port']),
        #         ssh_username=redcap_config.ssh['username'],
        #         ssh_password=redcap_config.ssh['password'],
        #         local_bind_address=(redcap_config.ssh['hostname'], redcap_config.mysql['port'] + 1),
        #         remote_bind_address=(redcap_config.mysql['hostname'], redcap_config.mysql['port'])
        #     )
        #     host = redcap_config.ssh['hostname']
        #     port = redcap_config.mysql['port'] + 1
        # else:
        #     self.tunnel = None
        host = redcap_config.mysql['hostname']
        port = redcap_config.mysql['port']

        uri = 'mysql+pymysql://%s:%s@%s:%s/%s' % (redcap_config.mysql['username'],
                                                  redcap_config.mysql['password'],
                                                  host,
                                                  port,
                                                  redcap_config.mysql['database'])

        try:
            self.engine = create_engine(uri)
        except ImportError as e:
            logging.critical('There is a problem with your MySQL settings, perhaps you need to set an env variable')
            logging.critical('e.g. export DYLD_LIBRARY_PATH="$DYLD_LIBRARY_PATH:/usr/local/Cellar/mysql/5.7.10/lib"')
            logging.critical(e.message)
            quit()

        self.engine.echo = False
        self.session = sessionmaker(bind=self.engine)()
        self.projects = redcap_config.projects

    def log_project_data(self, project, field_name=None):
        for doc in self.get_project_data(project, field_name):
            logging.info(doc)

    @log_event()
    def get_record_ids(self, project):
        project_id = self.projects[project]["project_id"]
        return self.session.query(RedCapData).unique(RedCapData.project_id == project_id).all()

    @log_event()
    def get_project_data(self, project, field_name=None):
        project_id = self.projects[project]["project_id"]
        if field_name:
            query = (RedCapData.project_id == project_id, RedCapData.field_name == field_name)
        else:
            query = (RedCapData.project_id == project_id,)
        return self.session.query(RedCapData).filter(*query).all()

    @log_event()
    def get_records(self, project):
        data = self.get_project_data(project)
        records = defaultdict(dict)
        for r in data:
            records[r.record][r.field_name] = r.value
            records[r.record]['event_id'] = r.event_id
        return records

    def get_scripted_experiments(self):
        scripted_experiments = self.get_records('scripted_experiments')
        annotation_data = self.get_records('annotations')
        for e in scripted_experiments.values():
            if 'start_time' not in e or 'end_time' not in e:
                continue
            start = pytz.UTC.localize(parse(e['start_time']) +
                                      timedelta(milliseconds=int(e['start_time_milliseconds'])))
            end = pytz.UTC.localize(parse(e['end_time']) +
                                    timedelta(milliseconds=int(e['end_time_milliseconds'])))

            annotations = {}
            for v in annotation_data.values():
                if v['experiment_id'] == e['record_id']:
                    annotations[v['annotator_id']] = {
                        'filename': v['annotation_file_name'],
                        'offset': v['offset'] if 'offset' in v else None
                    }

            yield ExperimentConfig(experiment_id=None,
                                   experiment_start=start,
                                   experiment_end=end,
                                   input_path=None,
                                   output_path=None,
                                   annotations=annotations)
        self.session.commit()

    # def __del__(self):
    #     if self.tunnel:
    #         self.tunnel.stop()

==> sphere_connector-0.5.4.tar.gz: sphere_connector-0.5.4/sphere_connector/redcap/redcap_client.py <==
"""
The MIT License (MIT)
Copyright (c) 2014-2015 University of Bristol

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
OR OTHER DEALINGS IN THE SOFTWARE.
"""
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
from sqlalchemy import Column, Integer, String, Text
# from wtforms_alchemy import ModelForm
# from sshtunnel import SSHTunnelForwarder
import logging
from collections import defaultdict
from datetime import timedelta
import pytz
from dateutil.parser import parse

from ..utils import log_event
from ..config import ExperimentConfig

Base = declarative_base()


class RedCapData(Base):
    __tablename__ = 'redcap_data'
    project_id = Column(Integer, nullable=False, default=0, primary_key=True)
    event_id = Column(Integer, primary_key=True)
    record = Column(String(100), primary_key=True)
    field_name = Column(String(100), primary_key=True)
    value = Column(Text)

    def __repr__(self):
        return "<RedCapData(project_id='%s', event_id='%s', record='%s', field_name='%s', value='%s'>" % \
               (self.project_id, self.event_id, self.record, self.field_name, self.value)


# class RedCapDataForm(ModelForm):
#     class Meta:
#         model = RedCapData


class RedCapClient(object):
    def __init__(self, redcap_config):
        logging.debug(redcap_config)

        # if redcap_config.ssh:
        #     self.tunnel = SSHTunnelForwarder(
        #         (redcap_config.ssh['hostname'], redcap_config.ssh['port']),
        #         ssh_username=redcap_config.ssh['username'],
        #         ssh_password=redcap_config.ssh['password'],
        #         local_bind_address=(redcap_config.ssh['hostname'], redcap_config.mysql['port'] + 1),
        #         remote_bind_address=(redcap_config.mysql['hostname'], redcap_config.mysql['port'])
        #     )
        #     host = redcap_config.ssh['hostname']
        #     port = redcap_config.mysql['port'] + 1
        # else:
        #     self.tunnel = None
        host = redcap_config.mysql['hostname']
        port = redcap_config.mysql['port']

        uri = 'mysql+pymysql://%s:%s@%s:%s/%s' % (redcap_config.mysql['username'],
                                                  redcap_config.mysql['password'],
                                                  host,
                                                  port,
                                                  redcap_config.mysql['database'])

        try:
            self.engine = create_engine(uri)
        except ImportError as e:
            logging.critical('There is a problem with your MySQL settings, perhaps you need to set an env variable')
            logging.critical('e.g. export DYLD_LIBRARY_PATH="$DYLD_LIBRARY_PATH:/usr/local/Cellar/mysql/5.7.10/lib"')
            logging.critical(e.message)
            quit()

        self.engine.echo = False
        self.session = sessionmaker(bind=self.engine)()
        self.projects = redcap_config.projects

    def log_project_data(self, project, field_name=None):
        for doc in self.get_project_data(project, field_name):
            logging.info(doc)

    @log_event()
    def get_record_ids(self, project):
        project_id = self.projects[project]["project_id"]
        return self.session.query(RedCapData).unique(RedCapData.project_id == project_id).all()

    @log_event()
    def get_project_data(self, project, field_name=None):
        project_id = self.projects[project]["project_id"]
        if field_name:
            query = (RedCapData.project_id == project_id, RedCapData.field_name == field_name)
        else:
            query = (RedCapData.project_id == project_id,)
        return self.session.query(RedCapData).filter(*query).all()

    @log_event()
    def get_records(self, project):
        data = self.get_project_data(project)
        records = defaultdict(dict)
        for r in data:
            records[r.record][r.field_name] = r.value
            records[r.record]['event_id'] = r.event_id
        return records

    @staticmethod
    def check_for_times(e):
        for prop in ['start_time', 'end_time', 'start_time_milliseconds', 'end_time_milliseconds']:
            if prop not in e:
                logging.debug('%s not in experiment %s' % (prop, e['record_id']))
                return False
        return True

    def get_experiments(self, basic_config, experiment_type, annotated=False):
        scripted_experiments = self.get_records(experiment_type + '_experiments')
        annotation_data = self.get_records('annotations')
        for e in scripted_experiments.values():
            if not self.check_for_times(e):
                continue

            start = pytz.UTC.localize(parse(e['start_time']) +
                                      timedelta(milliseconds=int(e['start_time_milliseconds'])))
            end = pytz.UTC.localize(parse(e['end_time']) +
                                    timedelta(milliseconds=int(e['end_time_milliseconds'])))

            annotations = {}
            if annotated:
                for v in annotation_data.values():
                    if v['experiment_id'] == e['record_id']:
                        annotations[v['annotator_id']] = {
                            'filename': v['annotation_file_name'],
                            'offset': v['offset'] if 'offset' in v else None
                        }

            yield ExperimentConfig(experiment_id=e['record_id'],
                                   experiment_start=start,
                                   experiment_end=end,
                                   annotation_path=basic_config.annotation_path,
                                   output_path=basic_config.output_path,
                                   annotations=annotations)
        self.session.commit()

    # def __del__(self):
    #     if self.tunnel:
    #         self.tunnel.stop()
